# matrix-appservice-kakaotalk - A Matrix-KakaoTalk puppeting bridge.
# Copyright (C) 2022 Tulir Asokan, Andrew Ferrazzutti
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
from typing import ClassVar, Optional

from attr import dataclass, asdict
import bson

from mautrix.types import SerializableAttrs, JSON


@dataclass(frozen=True)
class Long(SerializableAttrs):
    high: int
    low: int
    unsigned: bool

    @classmethod
    def from_bytes(cls, raw: bytes) -> "Long":
        return cls(**bson.loads(raw))

    @classmethod
    def from_optional_bytes(cls, raw: bytes | None) -> Optional["Long"]:
        return cls(**bson.loads(raw)) if raw is not None else None

    @classmethod
    def to_optional_bytes(cls, value: Optional["Long"]) -> bytes | None:
        return bytes(value) if value is not None else None

    def serialize(self) -> JSON:
        data = super().serialize()
        data["__type__"] = "Long"
        return data

    def __bytes__(self) -> bytes:
        return bson.dumps(asdict(self))

    def __int__(self) -> int:
        # TODO Is this right?
        return self.high << 32 + self.low

    def __str__(self) -> str:
        return f"{self.high << 32 if self.high else ''}{self.low}"

    ZERO: ClassVar["Long"]


Long.ZERO = Long(0, 0, False)
